from __future__ import annotations

from pathlib import Path

from configclasses import configclass
from rich.prompt import Prompt


@configclass
class Config:
    api_id: int
    api_hash: str

    @classmethod
    def create(cls, config_file_path: str | Path) -> Config:
        api_id = Prompt.ask("Enter your Telegram API id")
        api_hash = Prompt.ask("Enter your Telegram API hash")

        config = Config(api_id, api_hash)
        with open(config_file_path, "w") as config_file:
            for field in config.__dataclass_fields__:
                value = getattr(config, field)
                config_file.write(f"{field}={value}")
                config_file.write("\n")

        return config



import logging
from dataclasses import dataclass, asdict
from json import load, dump
from pathlib import Path
from typing import List

from rich import print
from rich.prompt import Confirm, Prompt
from telethon import TelegramClient
from telethon.errors import FloodWaitError

from telegram_bell.exceptions import SpecifiedChannelDoesNotExist

log = logging.getLogger("rich")


@dataclass
class SubscribedChannel:
    name: str
    search_keywords: List[str]
    last_id: int = 0

    def update(self, new_last_id: int, channels_file_path: str):
        self.last_id = new_last_id

        with open(channels_file_path, "r+") as file:
            channels = load(file)
            for channel in channels:
                if channel["name"] == self.name:
                    channel["last_id"] = self.last_id
            file.seek(0)
            dump(channels, file)
            file.truncate()

    @classmethod
    def read_from_json(
        cls, channels_file_path: str | Path
    ) -> List["SubscribedChannel"]:
        with open(channels_file_path, "r") as file:
            channels = load(file)
        return [SubscribedChannel(**channel) for channel in channels]

    @classmethod
    def create_config_file(cls, channels_file_path: str | Path):
        channels = []
        while Confirm.ask("Do you want to add a channel?"):
            name = Prompt.ask("Enter the name of the Telegram channel")
            search_keywords = Prompt.ask(
                f"Enter your search keywords for the channel '{name}'"
            ).split()
            channels.append(SubscribedChannel(name, search_keywords))

        with open(channels_file_path, "w") as channels_file:
            channels = [asdict(channel) for channel in channels]
            dump(channels, channels_file)


def keywords_in_message(keywords: List[str], text: str):
    for keyword in keywords:
        if keyword in text.lower():
            return True


async def read_messages_from_channel(
    client: TelegramClient, channel: SubscribedChannel, channels_file_path: str | Path
):
    reverse = channel.last_id != 0

    try:
        messages = await client.get_messages(
            channel.name, min_id=channel.last_id, reverse=reverse
        )
    except (ValueError, FloodWaitError) as e:
        log.exception(e)
        raise SpecifiedChannelDoesNotExist(
            "Specified channel doesn't exist. Check your config."
        )
    else:
        await handle_messages(channel, channels_file_path, messages)


async def handle_messages(channel, channels_file_path, messages):
    for message in messages:
        if channel.search_keywords:
            if message.text and keywords_in_message(
                channel.search_keywords, message.text
            ):
                print(message.sender_id, ":", message.text)
                await message.forward_to("me")
        else:
            print(message.sender_id, ":", message.text)
            await message.forward_to("me")
        channel.update(message.id, channels_file_path)



import logging
import struct
from pathlib import Path

import asyncclick as click
from configclasses.configclasses import dump
from configclasses.exceptions import ConfigFilePathDoesNotExist
from rich.prompt import Confirm
from rich.traceback import install
from telethon import TelegramClient

from telegram_bell.config import Config
from telegram_bell.exceptions import BadAPIConfiguration
from telegram_bell.notifier import SubscribedChannel, read_messages_from_channel

SESSION_NAME = "session"

log = logging.getLogger("rich")

app_path = Path("~/telegram_bell").expanduser()
app_path.mkdir(exist_ok=True)
config_path = app_path / ".env"
channels_file_path = app_path / "subscribed_channels.json"


async def setup_telegram_session(config: Config):
    client = TelegramClient(SESSION_NAME, config.api_id, config.api_hash)

    try:
        async with client:
            subscribed_channels = SubscribedChannel.read_from_json(channels_file_path)
            for channel in subscribed_channels:
                await read_messages_from_channel(
                    client, channel, channels_file_path
                )
    except struct.error:
        raise BadAPIConfiguration("Execute 'tbell config' with another parameters.")


async def setup_config():
    config = Config.create(config_path)
    SubscribedChannel.create_config_file(channels_file_path)
    await setup_telegram_session(config)


@click.group(chain=True)
async def cli():
    pass


@click.command(name="run")
async def run_command():
    try:
        config = Config.from_path(config_path)
    except ConfigFilePathDoesNotExist:
        config = Config.create(config_path)
        dump(config, config_path)
    client = TelegramClient(SESSION_NAME, config.api_id, config.api_hash)

    try:
        async with client:
            subscribed_channels = SubscribedChannel.read_from_json(channels_file_path)
            while True:
                for channel in subscribed_channels:
                    await read_messages_from_channel(
                        client, channel, channels_file_path
                    )
    except struct.error:
        raise BadAPIConfiguration("Connection error. Please, execute 'tbell config'.")


@click.command(name="config")
async def config_command():
    if Path(config_path).exists() and not Confirm.ask(
        "Config already exist, do you want to create a new one?"
    ):
        return

    await setup_config()


install()
cli.add_command(run_command)
cli.add_command(config_command)

if __name__ == "__main__":
    cli(_anyio_backend="asyncio")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['telegram_bell']

package_data = \
{'': ['*']}

install_requires = \
['12factor-configclasses>=1.0.0,<2.0.0',
 'anyio>=3.3.4,<4.0.0',
 'asyncclick>=8.0.1,<9.0.0',
 'python-dotenv>=0.19.1,<0.20.0',
 'rich>=10.12.0,<11.0.0',
 'telethon>=1.23,<2.0']

entry_points = \
{'console_scripts': ['tbell = telegram_bell.bell:cli']}

setup_kwargs = {
    'name': 'telegram-bell',
    'version': '0.3.3',
    'description': 'Notify you when something is mentioned in a telegram channel',
    'long_description': "# telegram-bell\n\nNotify you when something is mentioned in a telegram channel.\n\n## Install\n\n    pip install telegram-bell\n\n# Usage\n\n    tbell run\n\nBefore you can use, it will ask you for:\n    \n- your Telegram API ID\n- your Telegram API hash\n- channels and keywords which you want to get notified\n\nThen, the app will resend you the coinciding messages to your 'Saved Messages' channel\nin Telegram.",
    'author': 'Pablo Cabezas',
    'author_email': 'headsrooms@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
